@@ -53,13 +53,13 @@
 #define NETDATA_CHART_PRIO_SYSTEM_SOFT_INTR           1100 // freebsd only
 #define NETDATA_CHART_PRIO_SYSTEM_ENTROPY             1000
 #define NETDATA_CHART_PRIO_SYSTEM_UPTIME              1000
-#define NETDATA_CHART_PRIO_SYSTEM_IPC_MSQ_QUEUES       990 // freebsd only
-#define NETDATA_CHART_PRIO_SYSTEM_IPC_MSQ_MESSAGES    1000
-#define NETDATA_CHART_PRIO_SYSTEM_IPC_MSQ_SIZE        1100
-#define NETDATA_CHART_PRIO_SYSTEM_IPC_SEMAPHORES      1000
-#define NETDATA_CHART_PRIO_SYSTEM_IPC_SEM_ARRAYS      1000
-#define NETDATA_CHART_PRIO_SYSTEM_IPC_SHARED_MEM_SEGS 1000 // freebsd only
-#define NETDATA_CHART_PRIO_SYSTEM_IPC_SHARED_MEM_SIZE 1000 // freebsd only
+#define NETDATA_CHART_PRIO_SYSTEM_IPC_MSQ_QUEUES      1200 // freebsd only
+#define NETDATA_CHART_PRIO_SYSTEM_IPC_MSQ_MESSAGES    1201
+#define NETDATA_CHART_PRIO_SYSTEM_IPC_MSQ_SIZE        1202
+#define NETDATA_CHART_PRIO_SYSTEM_IPC_SEMAPHORES      1203
+#define NETDATA_CHART_PRIO_SYSTEM_IPC_SEM_ARRAYS      1204
+#define NETDATA_CHART_PRIO_SYSTEM_IPC_SHARED_MEM_SEGS 1205
+#define NETDATA_CHART_PRIO_SYSTEM_IPC_SHARED_MEM_SIZE 1206
 #define NETDATA_CHART_PRIO_SYSTEM_PACKETS             7001 // freebsd only
 
 
