@@ -25,7 +25,7 @@ run ./configure \
 run sed -i -e "s/curl_LDFLAGS =/curl_LDFLAGS = -all-static/" src/Makefile
 
 run make clean
-run make -j${SYSTEM_CPUS}
+run make -j$(find_processors)
 run make install
 
 if [ ${NETDATA_BUILD_WITH_DEBUG} -eq 0 ]
