@@ -3,7 +3,7 @@
 
 . $(dirname "${0}")/../functions.sh "${@}" || exit 1
 
-fetch "fping-4.0" "https://github.com/schweikert/fping/releases/download/v4.0/fping-4.0.tar.gz"
+fetch "fping-4.2" "https://github.com/schweikert/fping/releases/download/v4.2/fping-4.2.tar.gz"
 
 export CFLAGS="-static"
 
@@ -20,7 +20,7 @@ install:
 EOF
 
 run make clean
-run make -j${SYSTEM_CPUS}
+run make -j$(find_processors)
 run make install
 
 if [ ${NETDATA_BUILD_WITH_DEBUG} -eq 0 ]
