@@ -797,23 +797,23 @@ inline int web_client_api_request_v1(RRDHOST *host, struct web_client *w, char *
     }
 
     // get the command
-    char *tok = mystrsep(&url, "?");
-    if(tok && *tok) {
-        debug(D_WEB_CLIENT, "%llu: Searching for API v1 command '%s'.", w->id, tok);
-        uint32_t hash = simple_hash(tok);
+    if(url) {
+        debug(D_WEB_CLIENT, "%llu: Searching for API v1 command '%s'.", w->id, url);
+        uint32_t hash = simple_hash(url);
 
         for(i = 0; api_commands[i].command ;i++) {
-            if(unlikely(hash == api_commands[i].hash && !strcmp(tok, api_commands[i].command))) {
+            if(unlikely(hash == api_commands[i].hash && !strcmp(url, api_commands[i].command))) {
                 if(unlikely(api_commands[i].acl != WEB_CLIENT_ACL_NOCHECK) &&  !(w->acl & api_commands[i].acl))
                     return web_client_permission_denied(w);
 
-                return api_commands[i].callback(host, w, url);
+                //return api_commands[i].callback(host, w, url);
+                return api_commands[i].callback(host, w, (w->decoded_query_string + 1));
             }
         }
 
         buffer_flush(w->response.data);
         buffer_strcat(w->response.data, "Unsupported v1 API command: ");
-        buffer_strcat_htmlescape(w->response.data, tok);
+        buffer_strcat_htmlescape(w->response.data, url);
         return 404;
     }
     else {
